/*-----------------------------------------------------------------------*
  Project:  Nintendo Augmented Reality Library.
  File:     narEdgeLine.h

  Copyright (C)2011-2012 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensed developers
  and are protected by national and international copyright laws. They
  may not be disclosed to third parties or copied or duplicated in any
  form, in whole or in part, without the prior written consent of
  Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
*-----------------------------------------------------------------------*/

//
/* Please see man pages for details 
 
 
 */
#ifndef NAR_EDGE_LINE_H__
#define NAR_EDGE_LINE_H__

#include "narVec2.h"

namespace mw { namespace nar
{
    /* Please see man pages for details 
     
     */
    class EdgeLine_cl
    {
    public:
        EdgeLine_cl()
        {
            Init();
        }

        void Init();

        /* Please see man pages for details 
         */
        bool GetIntersection( Vec2F_st * p_Point, const EdgeLine_cl & r_Edge ) const;

        /* Please see man pages for details 
         */
        void Extend( const EdgeLine_cl & r );
        /* Please see man pages for details 
         */
        void ExtendStart( const EdgeLine_cl & r );
        /* Please see man pages for details 
         */
        void ExtendEnd( const EdgeLine_cl & r );
        
        f32   GetA() const { return m_A; }
        f32   GetB() const { return m_B; }
        f32   GetC() const { return m_C; }

        /* Please see man pages for details 
         */
        void SetStart( s32 x, s32 y )
        {
            m_Start.Set( x, y );
        }

        /* Please see man pages for details 
         */
        void SetStart( const Vec2I_st & cr )
        {
            SetStart( cr.X(), cr.Y() );
        }

        /* Please see man pages for details 
         */
        void SetEnd( s32 x, s32 y )
        {
            m_End.Set( x, y );
        }

        /* Please see man pages for details 
         */
        void SetEnd( const Vec2I_st & cr )
        {
            SetEnd( cr.X(), cr.Y() );
        }

        /* Please see man pages for details 
         */
        s32  GetSquareLength() const
        {
            return GetrStart().GetSquareDistance( GetrEnd() );
        }

        /* Please see man pages for details 
         */
        void AddSample( s32 x, s32 y );
        /* Please see man pages for details 
         */
        void AddSample( const Vec2I_st & cr )
        {
            AddSample( cr.X(), cr.Y() );
        }

        /* Please see man pages for details 
         */
        bool CalcEquation();
        /* Please see man pages for details 
         
         */
        bool SolveLinearLeastSquares();
        /* Please see man pages for details 
         
         */
        f32 GetDistance( const Vec2F_st &r ) const;
        /* Please see man pages for details 
         
         */
        f32 GetDistance( const Vec2I_st &r ) const;

        /* Please see man pages for details 
         */
        bool CheckSameEdgeDir( const EdgeLine_cl & cr, f32 * p_Sin, s32 * p_Dot ) const;

        /* Please see man pages for details 
         */
        bool CheckSameEdge( const EdgeLine_cl & cr, f32 * p_Sin, s32 * p_Dot ) const;

    private:
        /// a from straight-line equation:  ax + by + c = 0
        f32         m_A;
        /// b from straight-line equation:  ax + by + c = 0
        f32         m_B;
        /// c from straight-line equation:  ax + by + c = 0
        f32         m_C;

        Vec2I_st    m_Start;
        Vec2I_st    m_End;
        s32         m_SampleNum;
        s32         m_SumSquareXi;
        s32         m_SumSquareYi;
        s32         m_SumXi;
        s32         m_SumYi;
        s32         m_SumXiYi;

        const Vec2I_st & GetrStart() const { return m_Start; }
        const Vec2I_st & GetrEnd()  const { return m_End; }
        void addSums( const EdgeLine_cl & cr );
    };
}
}
#endif
//
