/*-----------------------------------------------------------------------*
  Project:  Nintendo Augmented Reality Library.
  File:     narImage.h

  Copyright (C)2011-2012 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensed developers
  and are protected by national and international copyright laws. They
  may not be disclosed to third parties or copied or duplicated in any
  form, in whole or in part, without the prior written consent of
  Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
*-----------------------------------------------------------------------*/

/* Please see man pages for details 
 
 
 */

#ifndef NAR_IMAGE_H__
#define NAR_IMAGE_H__

#include "narPort.h"
#include "narMarkerTemplate.h"
#include "narMesh.h"

namespace mw { namespace nar
{
    struct Color_st;

    /* Please see man pages for details 
     */
    enum Border_en
    {
        e_BorderWhiteBlack,	//
        e_BorderBlackWhite,	//
        e_BorderEnd
    };
    
    /* Please see man pages for details 
     */
    class Image_cl
    {
    public:
        /// X-direction step value when detecting outline
        static const u8 msc_DefaultXStep = 8;
        /// Y-direction step value when detecting outline
        static const u8 msc_DefaultYStep = 16;
        
        /* Please see man pages for details 
         
         
         
         
         */
        Image_cl( u16 w, u16 h, u32 * p_Work, u16 * p_DebugImg = 0 )
             : m_Width( w ), m_Height( h ), m_XStep( msc_DefaultXStep ), m_YStep( msc_DefaultYStep ), m_Border( 0 ), mcp_Image( 0 ), mp_CheckFlags( p_Work )
#if NAR_DEBUG__
               , mp_DebugImage( p_DebugImg )
#endif
        {
            NAR_NULL_TASSERT( p_Work );
#if ! NAR_DEBUG__
            ( ( void )&p_DebugImg );
#endif
        }

        /* Please see man pages for details 
         
         
         */
        void SetImage( const u16 * cp_Image )
        {
            NAR_NULL_TASSERT( cp_Image );
            mcp_Image = cp_Image;
        }

        /* Please see man pages for details 
         
         */
        u16 GetHeight() const { return m_Height; }

        /* Please see man pages for details 
         
         */
        u16 GetWidth()  const { return m_Width; }

        /* Please see man pages for details 
         
         
         */
        u8  GetXStep()  const { return m_XStep; }

        /* Please see man pages for details 
         
         
         */
        u8  GetYStep()  const { return m_YStep; }

        /* Please see man pages for details 
         
         
         
         
         
         
         */
        void    SetXStep( u8 x )
        {
            NAR_MIN_TASSERT( x, 1 );
            m_XStep = x;
        }

        /* Please see man pages for details 
         
         
         
         
         
         
         */
        void    SetYStep( u8 y )
        {
            NAR_MIN_TASSERT( y, 1 );
            m_YStep = y;
        }
        
        /* Please see man pages for details 
         
         
         
         */
        u16     GetColor( u32 x, u32 y ) const
        {
#if NAR_PIXEL_FMT == NAR_PIXEL_FMT_RGB555
            return static_cast< u16 >( mcp_Image[ y * m_Width + x ] );
#endif
#if NAR_PIXEL_FMT == NAR_PIXEL_FMT_YUV422_BATCH
            int pos = y * m_Width + x;
            u8 Y = static_cast< u8 >( mcp_Image[ pos ] & 0xff );
            u8 U = static_cast< u8 >( mcp_Image[ pos & ~1 ] >> 8 & 0xff );
            u8 V = static_cast< u8 >( mcp_Image[ ( pos & ~1 ) + 1 ] >> 8 & 0xff );
            return ConvYUVtoRGB5551( Y, U, V );
#endif
        }

        /* Please see man pages for details 
         */
        void		SetBorder( Border_en b )
        {
            m_Border	= b;
        }

        /* Please see man pages for details 
         */
        Border_en	GetBorder()
        {
            return static_cast< Border_en >( m_Border );
        }

        /* Please see man pages for details 
         
         
         
         */
        template< MeshDiv_e N >
        void     PickupMeshData( const Vec2F_st * cap_Points, MeshColorData_tc< N > & r_MeshColorData ) const
        {
            // Calculate the mesh from 4 points
            MeshStrictWork_tc< N > work;
            MeshStrict_cl mesh( work );
            mesh.precalc( cap_Points );
#if NAR_DEBUG__
            //  Debug information display
            mesh.drawInfo( *this, cap_Points, false );
#endif
            mesh.pickupFast( *this, r_MeshColorData );
        }

        /* Please see man pages for details 
         
         
         
         
         */
        template< MeshDiv_e N >
        void     PickupMeshData( const Vec2F_st * cap_Points, MeshColorData_tc< N > & r_MeshColorData, MeshVtxData_tc< N > & r_MeshVtxData ) const
        {
            // Calculate the mesh from 4 points
            MeshStrictWork_tc< N > work;
            MeshStrict_cl mesh( work );
            mesh.precalc( cap_Points );
#if NAR_DEBUG__
            //  Debug information display
            mesh.drawInfo( *this, cap_Points, false );
#endif
            mesh.pickupFast( *this, r_MeshColorData, r_MeshVtxData );
        }

        //
        /* Please see man pages for details 
         
         
         */
        void     GetAvgColor( u32 x, u32 y, Color_st * p_Color ) const;

        /* Please see man pages for details 
         
         
         */
        void     GetAvgColorFast( u32 x, u32 y, Color_st * p_Color ) const;

        /* Please see man pages for details 
         
         */
        static u8 GetR( u16 color )
        {
            return static_cast< u8 >( color >> NAR_RGB5A1_R_SHIFT & 0x1f );
        }

        /* Please see man pages for details 
         
         */
        static u8 GetG( u16 color )
        {
            return static_cast< u8 >( color >> NAR_RGB5A1_G_SHIFT & 0x1f );
        }

        /* Please see man pages for details 
         
         */
        static u8 GetB( u16 color )
        {
            return static_cast< u8 >( color >> NAR_RGB5A1_B_SHIFT & 0x1f );
        }

        /* Please see man pages for details 
         */
        static u8 GetSimpleYRaw( u8 r, u8 g, u8 b )
        {
            return static_cast< u8 >( r + ( g << 1 ) + b );
        }

        /* Please see man pages for details 
         */
        u8 GetSimpleYRaw( u32 x, u32 y ) const
        {
#if NAR_PIXEL_FMT == NAR_PIXEL_FMT_RGB555
            u16 color = GetColor( x, y );
            return GetSimpleYRaw( GetR( color ), GetG( color ), GetB( color ) );
#endif
#if NAR_PIXEL_FMT == NAR_PIXEL_FMT_YUV422_BATCH
            u8 ret = static_cast< u8 >( mcp_Image[ y * m_Width + x ] & 0xff );
            if ( m_Border )
                return ~ret;
            else
                return ret;
#endif
        }

        /* Please see man pages for details 
         */
        void Checked( u32 x, u32 y )
        {
            register u32 pixelNo    = y * m_Width + x;
            register u32 flagNo     = pixelNo >> 5;
            register u32 bitNo      = pixelNo & 0x1f;
            mp_CheckFlags[ flagNo ] |= ( 1 << bitNo );
        }

        /* Please see man pages for details 
         */
        bool IsChecked( u32 x, u32 y ) const
        {
            register u32 pixelNo    = y * m_Width + x;
            register u32 flagNo     = pixelNo >> 5;
            register u32 bitNo      = pixelNo & 0x1f;
            return 0 != ( mp_CheckFlags[ flagNo ] & ( 1 << bitNo ) );
        }

        /* Please see man pages for details 
         */
        void ClearChecked()
        {
            ClearFast( mp_CheckFlags, static_cast< u32 >( (( m_Width * m_Height + 31 ) & ~31) >> 3 ) );
        }

        /* Please see man pages for details 
         */
        static u16  ConvYUVtoRGB5551( u8 y, u8 u, u8 v );


#if NAR_DEBUG__
#if NAR_CTR_NATIVE_DEBUG_IMAGE
        /* Please see man pages for details 
         
         */
        u32     GetDmpNativeTexelPos( u32 x, u32 y ) const;
#endif


        /* Please see man pages for details 
         
         
         */
        void    SetColorDebug( u32 x, u32 y, u16 color ) const
        {
            if ( mp_DebugImage && x < m_Width && y < m_Height )
            {
#if NAR_CTR_NATIVE_DEBUG_IMAGE
                int idx = GetDmpNativeTexelPos( x, y );
#else
                int idx = y * m_Width + x;
#endif
                if ( 0 == mp_DebugImage[ idx ] )
                    mp_DebugImage[ idx ] = color;
            }
        }

        /* Please see man pages for details 
         */
        void    ClearDebugImage()
        {
            if ( mp_DebugImage )
                ClearFast( mp_DebugImage, m_Width * m_Height * sizeof( u16 ) );
        }

        /* Please see man pages for details 
         */
        void    DrawLineSegmentOnDebug( const Vec2F_st & cr_Start, const Vec2F_st & cr_End, u16 color ) const;

        /* Please see man pages for details 
         */
        void    DrawLineOnDebug( f32 a, f32 b, f32 c, u16 color ) const;

        /* Please see man pages for details 
         */
        void    DrawArrowOnDebug( const Vec2I_st & cr_Start, const Vec2I_st & cr_End, u16 color ) const;

        /* Please see man pages for details 
         */
        void    DrawPlusMarkOnDebug( const Vec2I_st & r_Point, u16 color ) const;

#endif
        //
        /* Please see man pages for details 
         */
        void    SetDebugImage( u16 * p_Image )
        {
#if NAR_DEBUG__
            mp_DebugImage   = p_Image;
#else
            ((void)&p_Image);
#endif
        }
    private:
        /// Image data width
        u16     m_Width;
        /// Image data height
        u16     m_Height;
        /// X-direction step width
        /// Related to the outline width in the image of the marker you wish to detect.
        u8      m_XStep;
        /// Y-direction step width
        /// Related to the Y-direction size in the image of the marker you wish to detect.
        u8      m_YStep;

        ///	Outline
        u8		m_Border;

        u8     	pad;
        
        /// Pointer to the image data
        const u16 * mcp_Image;
        /// Pixel check flag
        u32 *   mp_CheckFlags;
#if NAR_DEBUG__
        u16 *   mp_DebugImage;
#endif
    };

    /* Please see man pages for details 
     
     */
#define NAR_IMAGE_WORK_SIZE_4TH( w, h ) (((w)*(h)+31)>>5)
}
}

#endif
