/*-----------------------------------------------------------------------*
  Project:  Nintendo Augmented Reality Library.
  File:     narMarker.h

  Copyright (C)2011-2012 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensed developers
  and are protected by national and international copyright laws. They
  may not be disclosed to third parties or copied or duplicated in any
  form, in whole or in part, without the prior written consent of
  Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
*-----------------------------------------------------------------------*/
/* Please see man pages for details 
 
 
 */

#ifndef NAR_MARKER_H__
#define NAR_MARKER_H__

#include "narConfig.h"
#include "narPort.h"
#include "narVec2.h"
#include "narVec3.h"
#include "narProjection.h"
#include "narTransformation.h"
#include "narMarkerTemplate.h"
#include "narDistortion.h"

namespace mw { namespace nar
{
    class Distortion_cl;
    /* Please see man pages for details 
     */
    class Marker_cl
    {
    public:
        Marker_cl() : mcp_PatternVtxData( 0 )
        {}

        /* Please see man pages for details 
         
         */
        s32 GetID() const { return mcp_Template ? mcp_Template->id : -1; }

        /* Please see man pages for details 
         
         */
        f32 GetScore() const { return m_Score; }

        /* Please see man pages for details 
         
         */
        s32 GetDir() const { return m_Dir; }

        /* Please see man pages for details 
         */
        Transformation_cl & GetrTransformation() { return m_Trans; }

        /* Please see man pages for details 
         */
        bool IsEstimated() const { return m_Estimated; }

        /* Please see man pages for details 
         */
        f32 GetSideLength() const
        {
            return mcp_Template ? mcp_Template->sideLength : 0 ;
        }

        /* Please see man pages for details 
         */
        f32 GetAspectRatio() const
        {
            return mcp_Template ? mcp_Template->aspectRatio : 0 ;
        }

        /* Please see man pages for details 
         */
        const Vec3F_st & GetrNormal() { return m_Trans.GetrYAxis(); } 

        /* Please see man pages for details 
         
         
         */
        void SuppressShake( f32 sim, f32 far );

        /* Please see man pages for details 
         
         
         
         
         */
        const PatternVtxData_cl * GetpPatternVtxData()
        {
            return mcp_PatternVtxData;
        }

        //
        /* Please see man pages for details 
         */
        void Init( const MarkerTemplate_st * cp_Temp,
                   const Vec2F_st * cap_Points,
                   const PatternVtxData_cl * cp_PatternVtxData,
                   s32 dir,
                   f32 c_Score );


        ///
        void SetEstimated( bool b ) { m_Estimated = b; }

        const Vec2F_st * GetpDetectedVtx() const
        {
            return ma_VtxDetected;
        }

        void SetDetectedVtx( s32 idx, f32 x, f32 y )
        {
            ma_VtxDetected[ idx ].Set( x, y );
        }

        const Vec3F_st & GetrVtxOnRefPlane( s32 idx ) const
        {
            return ma_VtxOnRefPlane[ idx ];
        }
        
        Marker_cl * GetpPrev() { return mp_Prev; }
        void SetPrev( Marker_cl * p_Prev ) { mp_Prev = p_Prev; }
        Marker_cl * GetpNext() { return mp_Next; }
        void SetNext( Marker_cl * p_Next ) { mp_Next = p_Next; }

        /* Please see man pages for details 
         */
        f32 GetSumOfEachVtxSquareDistance( const Marker_cl & cr ) const;

        /* Please see man pages for details 
         */
        void AverageVtx( const Marker_cl & cr );

        /* Please see man pages for details 
         */
        void    CalcVtxOnRefPlane( const Projection_cl & cr_Proj )
        {
            for ( int vtx = 0; vtx < 4; vtx++ )
            {
                cr_Proj.GetVtxOnReferencePlane( ma_VtxOnRefPlane[ vtx ], ma_VtxDetected[ vtx ] );
            }
        }

        /* Please see man pages for details 
         */
        void    CalcVtxOnRefPlane( const Projection_cl & cr_Proj, const Distortion_cl & cr_Dist )
        {
            for ( int vtx = 0; vtx < 4; vtx++ )
            {
                Vec2F_st ideal;
                cr_Dist.GetIdealFromObserv( ideal, ma_VtxDetected[ vtx ] );
                cr_Proj.GetVtxOnReferencePlane( ma_VtxOnRefPlane[ vtx ], ideal );
            }
        }

        /* Please see man pages for details 
         */
        f32 EvaluateMatrix() const;

        /* Please see man pages for details 
         
         
         
         
         */
        f32 AdjustRotate( f32 targetErr, s32 maxCnt );

        /*  
*/
        Vec3F_st & GetrYAxisMove() { return m_YAxisMove; }

        ///
        void SetYAxisMoveRec( bool b ) { m_YAxisMoveRec = b; }

        /* Please see man pages for details 
         
         */
        bool IsSimilarYAxisRotate();

        f32     GetInterpolationCoefficient() const { return m_InterpolationCoefficient; }
        void    SetInterpolationCoefficient( f32 t ) { m_InterpolationCoefficient = t; }

        /// Square of the distance of the quadrupled midpoint
        f32 GetSqareDistanceQuadMid( const Vec2F_st * cap_Points );
        /// Square of the distance of the quadrupled midpoint
        f32 GetSqareDistanceQuadMid( const Marker_cl & cr )
        {
            return GetSqareDistanceQuadMid( cr.ma_VtxDetected );
        }
        
        /// 2 times the area
        static f32 GetDoubleArea( const Vec2F_st * cap_Points );
        /// 2 times the area
        f32 GetDoubleArea()
        {
            return GetDoubleArea( ma_VtxDetected );
        }
        
        /// Returns whether the outlines overlap
        bool IsOverlapped( const Vec2F_st * cap_Points )
        {
            return GetDoubleArea( cap_Points ) * 2.f > GetSqareDistanceQuadMid( cap_Points );
        }

        /// Sets interpolation flag
        void SetInterpolated( bool b ) { m_Interpolated = b; }
        
        //
    private:
        /// Detected vertex
        /// Stored in order: left top, left bottom, right bottom, right top
        Vec2F_st                    ma_VtxDetected[ 4 ];
        /// Vertex on reference plane
        Vec3F_st                    ma_VtxOnRefPlane[ 4 ];
        /// Score when detecting
        f32                         m_Score;
        /// Detected direction
        s32                         m_Dir;
        /// Pointer to template
        const MarkerTemplate_st *   mcp_Template;
        /// Conversion matrix
        Transformation_cl           m_Trans;

        /// Link to marker with same ID in image in previous frame
        Marker_cl *         mp_Prev;
        Marker_cl *         mp_Next;

        /// Whether attitude matrix was estimated
        bool                m_Estimated;

        /// Whether previous marker coordinate (0,1,0) movement was saved
        bool                m_YAxisMoveRec;

        bool                m_Interpolated;

        u8                  pad;

        /// Interpolation coefficient
        f32                 m_InterpolationCoefficient;

        /// Vector from previous marker coordinate (0,1,0) to current marker coordinate (0,1,0)
        Vec3F_st            m_YAxisMove;

        /// Pointer to the region for getting the pickup point
        const PatternVtxData_cl * mcp_PatternVtxData;
    };
}
}

#endif
