/*-----------------------------------------------------------------------*
  Project:  Nintendo Augmented Reality Library.
  File:     narMarkerPattern.h

  Copyright (C)2011-2012 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensed developers
  and are protected by national and international copyright laws. They
  may not be disclosed to third parties or copied or duplicated in any
  form, in whole or in part, without the prior written consent of
  Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
*-----------------------------------------------------------------------*/
/* Please see man pages for details 
 
 
 */
#ifndef NAR_MARKER_PATTERN_H__
#define NAR_MARKER_PATTERN_H__

#include "narPort.h"
#include "narVec2.h"
#include "narMesh.h"

namespace mw { namespace nar
{
    /* Please see man pages for details 
     */
    class PatternVtxData_cl : public MeshVtxData_tc< e_MeshDiv15 >
    {
    public:
        /* Please see man pages for details 
         */
        const Vec2F_st & GetDirVtx( s32 t, s32 s, s32 dir ) const;
    };

    //

    /// Number of marker sample pattern divisions (per side)
    const u8    c_MarkerDivide = 15;

    class   Image_cl;
    struct  Vec2F_st;
    struct  MarkerTemplate_st;
    struct  MarkerPatternWeight_st;

    /* Please see man pages for details 
     */
    class PatternColorData_cl : public MeshColorData_tc< e_MeshDiv15 >
    {
    public:
        /* Please see man pages for details 
         */
        const Color_st & GetColorDirected( s32 t, s32 s, s32 dir ) const;
    };

    /* Please see man pages for details 
     
     */
    struct MarkerPattern_st
    {
        const Color_st &
            GetColor( s32 t, s32 s ) const { return maa_MarkerDesign[ t ][ s ]; }
        
        Color_st &
            GetColor( s32 t, s32 s ) { return maa_MarkerDesign[ t ][ s ]; }

        /* Please see man pages for details 
         */
        void
            Extract( const Image_cl & cr_Image, const Vec2F_st * cap_Points, PatternVtxData_cl * p_VtxData );

        /* Please see man pages for details 
         
         */
        void
            ExtractFast( const Image_cl & cr_Image, const Vec2F_st * cap_Pointsm, PatternVtxData_cl * p_VtxData );

        /* Please see man pages for details 
         
         */
        void
            ExtractStrictly( const Image_cl & cr_Image, const Vec2F_st * cap_Points, PatternVtxData_cl * p_VtxData );

        /* Please see man pages for details 
         */
        f32
            GetCorrelation( const MarkerTemplate_st & cr, s32 dir ) const;

        /*  
*/
        bool
            ExtractForTemplate( const Image_cl & cr_Image, const Vec2F_st * cap_Points, MarkerPatternWeight_st & r_w );
        
        /* Please see man pages for details 
         */
        void
            Print() const;

        /// Pattern data
        Color_st
            maa_MarkerDesign[ c_MarkerDivide ][ c_MarkerDivide ];

        /// Offset
        s8
            m_Offset;

        /// Pattern value average
        f32
            m_Average;

        /// Pattern value standard deviation
        f32
            m_Stdevp;

    };

    /* Please see man pages for details 
     */
    class MarkerPatternData_cl : public MeshColorData_ac
    {
    public:
        MarkerPatternData_cl( MarkerPattern_st & r_Pattern )
            : mr_Pattern( r_Pattern )
        {}
        virtual const Color_st &
            GetColor( s32 t, s32 s ) const { return mr_Pattern.GetColor( t, s ); }
        
        virtual Color_st &
            GetColor( s32 t, s32 s ) { return mr_Pattern.GetColor( t, s ); }
    private:
        MarkerPattern_st & mr_Pattern;
    };
    //

}
}
#endif
