/*-----------------------------------------------------------------------*
  Project:  Nintendo Augmented Reality Library.
  File:     narMesh.h

  Copyright (C)2011-2012 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensed developers
  and are protected by national and international copyright laws. They
  may not be disclosed to third parties or copied or duplicated in any
  form, in whole or in part, without the prior written consent of
  Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
*-----------------------------------------------------------------------*/
/* Please see man pages for details 
 
 
 */
#ifndef NAR_MESH_H__
#define NAR_MESH_H__

#include "narPort.h"
#include "narVec2.h"

namespace mw { namespace nar
{
    class Image_cl;

    /* Please see man pages for details 
     
     */
    enum MeshDiv_e
    {
        /// 15
        e_MeshDiv15 = 15,
        /// 31 
        e_MeshDiv31 = 31, 
        /// 63
        e_MeshDiv63 = 63 
    };

    /* Please see man pages for details 
     
     
     */
    struct Color_st
    {
        s8 r,g,b;
    };

    /* Please see man pages for details 
     */
    class MeshColorData_ac
    {
    public:
        /* Please see man pages for details 
         */
        virtual const Color_st &
            GetColor( s32 t, s32 s ) const = 0;

        /* Please see man pages for details 
         */
        virtual Color_st &
            GetColor( s32 t, s32 s ) = 0;
    };

    /* Please see man pages for details 
     
     */
    template< MeshDiv_e N >
    class MeshColorData_tc : public MeshColorData_ac
    {
    public:
        /* Please see man pages for details 
         */
        virtual const Color_st &
            GetColor( s32 t, s32 s ) const { return maa_Data[ t ][ s ]; }

        /* Please see man pages for details 
         */
        virtual Color_st &
            GetColor( s32 t, s32 s ) { return maa_Data[ t ][ s ]; }
    protected:
        Color_st    maa_Data[ N ][ N ];
        s8          pad[ 5 ];
    };

    /* Please see man pages for details 
     */
    class MeshVtxData_ac
    {
    public:
        /* Please see man pages for details 
         */
        virtual const Vec2F_st &
            GetVtx( s32 t, s32 s ) const = 0;

        /* Please see man pages for details 
         */
        virtual Vec2F_st &
            GetVtx( s32 t, s32 s ) = 0;
    };

    /* Please see man pages for details 
     
     */
    template< MeshDiv_e N >
    class MeshVtxData_tc : public MeshVtxData_ac
    {
    public:
        /* Please see man pages for details 
         */
        virtual const Vec2F_st &
            GetVtx( s32 t, s32 s ) const { return maa_Data[ t ][ s ]; }

        /* Please see man pages for details 
         */
        virtual Vec2F_st &
            GetVtx( s32 t, s32 s ) { return maa_Data[ t ][ s ]; }
    protected:
        Vec2F_st    maa_Data[ N ][ N ];
    };

    //

    /* Please see man pages for details 
     */
    class MeshStrictWork_ac
    {
    public:
        virtual Vec2F_st &
            getGrid( s32 nEdge, s32 nDiv ) = 0;

        virtual const Vec2F_st &
            getGrid( s32 nEdge, s32 nDiv ) const = 0;

        virtual Vec2F_st *
            getpGrid( s32 nEdge ) = 0;

        virtual f32
            getST( s32 nST, s32 nDiv ) const = 0;

        virtual f32 *
            getpST( s32 nST ) = 0;

        virtual s32
            getDiv() const = 0;
    };

    /* Please see man pages for details 
     
     */
    template< s32 N >
    class MeshStrictWork_tc : public MeshStrictWork_ac
    {
    public:
        virtual Vec2F_st &
            getGrid( s32 nEdge, s32 nDiv ) { return maa_Grid[ nEdge ][ nDiv ]; }

        virtual const Vec2F_st &
            getGrid( s32 nEdge, s32 nDiv ) const { return maa_Grid[ nEdge ][ nDiv ]; }

        virtual Vec2F_st *
            getpGrid( s32 nEdge ) { return maa_Grid[ nEdge ]; }

        virtual f32
            getST( s32 nST, s32 nDiv ) const { return maa_ST[ nST ][ nDiv ]; }

        virtual f32 *
            getpST( s32 nST ) { return maa_ST[ nST ]; }

        virtual s32
            getDiv() const { return N; }

    private:
        Vec2F_st    maa_Grid[ 4 ][ N + 2 ];
        f32         maa_ST[ 2 ][ N + 2 ];
    };

    /* Please see man pages for details 
     */
    class MeshStrict_cl
    {
    public:
        MeshStrict_cl(  MeshStrictWork_ac & work ) : mr_Work( work )
        {}

        /// Precalculation
        void    precalc( const Vec2F_st * cap_Points );

#if NAR_DEBUG__
        /// Debug information display
        void    drawInfo( const Image_cl & cr_Image, const Vec2F_st * cap_Points, bool isMarker );
#endif
        /// Extract color
        void    pickup( const Image_cl & cr_Image, MeshColorData_ac & r_MeshColor );

        /// Extract color and coordinates
        void    pickup( const Image_cl & cr_Image, MeshColorData_ac & r_MeshColor, MeshVtxData_ac & r_MeshVtx );

        /// Extract color
        void    pickupFast( const Image_cl & cr_Image, MeshColorData_ac & r_MeshColor );

        /// Extract color and coordinates
        void    pickupFast( const Image_cl & cr_Image, MeshColorData_ac & r_MeshColor, MeshVtxData_ac & r_MeshVtx );

        /// Get offset calculated with the extraction procedure
        s8      getOffset() const { return m_Offset; }
    private:
        Vec2F_st            m_v10;
        Vec2F_st            m_v30;
        Vec2F_st            m_v21;
        Vec2F_st            m_v23;
        MeshStrictWork_ac & mr_Work;
        s8                  m_Offset;
        u8                  pad1[ 3 ];
#if NAR_DEBUG__
        bool                m_isCalcVp;
        bool                m_isDivEq;
        u8                  pad2[ 2 ];
#endif
        void    precalcVp( const Vec2F_st * cap_Points );
        void    precalcApprox( const Vec2F_st * cap_Points, const f32 * cap_L, const f32 * a_Cos );
    };
    //
}
}
#endif
