/*-----------------------------------------------------------------------*
  Project:  Nintendo Augmented Reality Library.
  File:     narVec2.h

  Copyright (C)2011-2012 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensed developers
  and are protected by national and international copyright laws. They
  may not be disclosed to third parties or copied or duplicated in any
  form, in whole or in part, without the prior written consent of
  Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
*-----------------------------------------------------------------------*/

//
/* Please see man pages for details 
 
 
 */

#ifndef NAR_VEC_2_H__
#define NAR_VEC_2_H__

#include "narPort.h"
#include "narMath.h"

namespace mw { namespace nar
{
    /* Please see man pages for details 
     
     */
    template < class T >
    struct Vec2_ts
    {
        Vec2_ts()
        {}

        Vec2_ts( T x, T y )
        {
            Set( x, y );
        }

        Vec2_ts( T val )
        {
            SetAll( val );
        }

        Vec2_ts( const Vec2_ts< T > & r )
        {
            Set( r );
        }

        void Set( const Vec2_ts< T > & r )
        {
            Set( r.X(), r.Y() );
        }

        void Set( T x, T y )
        {
            X() = x;
            Y() = y;
        }

        void SetAll( T val )
        {
            Set( val, val );
        }

        T & X() { return ma_[ me_X ]; }
        T & Y() { return ma_[ me_Y ]; }
        const T & X() const { return ma_[ me_X ]; }
        const T & Y() const { return ma_[ me_Y ]; }

        /* Please see man pages for details 
         */
        Vec2_ts & Add( T x, T y )
        {
            X() += x;
            Y() += y;
            return *this;
        }

        /* Please see man pages for details 
         */
        Vec2_ts & Add( const Vec2_ts< T > & r )
        {
            return Add( r.X(), r.Y() );
        }

        /* Please see man pages for details 
         */
        Vec2_ts operator+( const Vec2_ts< T > & r ) const
        {
            Vec2_ts ret( *this );
            return ret.Add( r );
        }

        /* Please see man pages for details 
         */
        Vec2_ts & Sub( T x, T y )
        {
            X() -= x;
            Y() -= y;
            return *this;
        }

        /* Please see man pages for details 
         */
        Vec2_ts & Sub( const Vec2_ts< T > & r )
        {
            return Sub( r.X(), r.Y() );
        }

        /* Please see man pages for details 
         */
        Vec2_ts & Scale( T a )
        {
            X() *= a;
            Y() *= a;
            return *this;
        }

        /* Please see man pages for details 
         */
        Vec2_ts operator*( T s ) const
        {
            Vec2_ts< T > ret( *this );
            return ret.Scale( s );
        }

        /* Please see man pages for details 
         */
        Vec2_ts & Div( T a )
        {
            T invA = static_cast< T >( 1 ) / a;
            return Scale( invA );
        }

        /* Please see man pages for details 
         */
        T   Dot( const Vec2_ts< T > & r ) const
        {
            return X() * r.X() + Y() * r.Y();
        }

        /* Please see man pages for details 
         */
        T   Cross( const Vec2_ts< T > & r ) const
        {
            return X() * r.Y() - Y() * r.X();
        }

        /* Please see man pages for details 
         */
        bool operator==( const Vec2_ts< T > & r ) const
        {
            return X() == r.X() && Y() == r.Y();
        }

        /* Please see man pages for details 
         */
        bool operator!=( const Vec2_ts< T > & r ) const
        {
            return ! *this == r;
        }

        /* Please see man pages for details 
         */
        bool Equal( T x, T y )
        {
            return X() == x && Y() == y;
        }

        /*  
*/
        void Rotate90()
        {
            Set( -Y(), X() );
        }

        /* Please see man pages for details 
         */
        T GetSquareLength() const
        {
            return Pow2( X() ) + Pow2( Y() );
        }

        /* Please see man pages for details 
         */
        T GetSquareDistance( const Vec2_ts< T > & r ) const
        {
            Vec2_ts< T > ret( r );
            ret.Sub( *this );
            return ret.GetSquareLength();
        }

    private:
        enum
        {
            me_X, me_Y, me_ArrayNum
        };

        T ma_[ me_ArrayNum ];
    };

    struct Vec2I_st;
    /* Please see man pages for details 
     
     */
    struct Vec2F_st : public Vec2_ts< f32 >
    {
        Vec2F_st()
        {}

        Vec2F_st( const Vec2_ts< f32 > & r )
        {
            Set( r.X(), r.Y() );
        }

        /* Please see man pages for details 
         */
        Vec2F_st( const Vec2I_st & r );

        Vec2F_st( int x, int y )
        {
            Set( static_cast< f32 >( x ), static_cast< f32 >( y ) );
        }

        Vec2F_st( u32 x, u32 y )
        {
            Set( static_cast< f32 >( x ), static_cast< f32 >( y ) );
        }

        Vec2F_st( f32 x, f32 y )
             : Vec2_ts< f32 >( x, y )
        {}

        /* Please see man pages for details 
         */
        bool Normalize()
        {
            f32 len = GetLength();
            if ( len < c_Epsilon ) return false;
            Scale( static_cast< f32 >( 1.0 ) / len );
            return true;
        }

        /* Please see man pages for details 
         */
        f32 GetLength()
        {
            return Sqrt( GetSquareLength() );
        }

    };

    /* Please see man pages for details 
     
     */
    struct Vec2I_st : public Vec2_ts< s32 >
    {
        Vec2I_st()
        {}

        Vec2I_st( s32 x )
             : Vec2_ts< s32 >( x )
        {}

        Vec2I_st( s32 x, s32 y )
             : Vec2_ts< s32 >( x, y )
        {}

        Vec2I_st & Scale( f32 a )
        {
            X() *= a;
            Y() *= a;
            return *this;
        }
    };

    /*  
*/
    inline Vec2F_st::Vec2F_st( const Vec2I_st & r )
    {
        Set( static_cast< f32 >( r.X() ),
             static_cast< f32 >( r.Y() ) );
    }

    template < class T >
    static inline Vec2_ts< T > operator+( const Vec2_ts< T > & r_A, const Vec2_ts< T > & r_B )
    {
        Vec2_ts< T > ret( r_A );
        return ret.Add( r_B );
    }

}
}
#endif
//
