/*-----------------------------------------------------------------------*
  Project:  Nintendo Augmented Reality Library.
  File:     sampledemos\marker_maker\main.cpp

  Copyright (C)2011 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensed developers
  and are protected by national and international copyright laws. They
  may not be disclosed to third parties or copied or duplicated in any
  form, in whole or in part, without the prior written consent of
  Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
*-----------------------------------------------------------------------*/

/* Please see man pages for details 
 
 
 
 */

#include <nn.h>

#include <nn/fnd.h>

#include <nn/gx.h>
#include <nn/math.h>
#include <nn/fs.h>

#include <nn/hid.h>
#include <nn/camera.h>
#include <nn/y2r.h>

#include "nar.h"

#include "cube.h"
#include "hatenaMarkerTemplate.h"

#include "narDemoConfig.h"
#include "narDemoCamera.h"
#include "narDemoPhoto.h"
#include "narDemoY2r.h"

#include "demo.h"

#define NEAR        (1)
#define FAR         (5000)

namespace
{
    //  Image processing work
    u32 sa_NarImageWork[ NAR_IMAGE_WORK_SIZE_4TH( IMG_WIDTH, IMG_HEIGHT ) ] ;

    //  Marker detection work
    mw::nar::MarkerDetectWork_cl        s_DetectWork;

    //  Marker data
    mw::nar::MarkerData_tc< 10 >        s_MarkerData;

    const int                           c_MakerWorkSize = 5;

    const int                           c_gl_memory_size = 0x800000;

    demo::RenderSystemDrawing           s_RenderSystem;

    nn::fnd::ExpHeap                    s_AppHeap;
    uptr                                s_HeapForGx;

    mw::nar::MarkerTrans_cl             s_NarTrans;
    mw::nar::Projection_cl              s_Projection;

    bool    s_ViewInfo      = false;
    bool    s_MarkerMaker   = false;
    u8      s_MarkerIdx     = 0;

    /* program id */
    GLuint s_PgID;

    /* shader id */
    GLuint s_ShID;

    //=====================================================================
    /*
     *  Marker
     */
    mw::nar::MarkerTemplate_st s_MarkerTemplate = g_HatenaMarker;

    //=====================================================================
    /* Please see man pages for details 
     
     */
    class CameraModule_cl
    {
    public:
        void    Initialize();
        /* Please see man pages for details 
         */
        bool    Wait();

        void    Finalize();

        CameraProcess_cl &  GetrCameraProcess() { return m_Process; }
    private:
        CameraProcess_cl    m_Process;
        nn::os::Event       m_EndEvent;
        u32                 pad;
    };
    //=====================================================================
    void CameraModule_cl::Initialize()
    {
        // Initialize CAMERA
        nn::camera::Initialize();

        // Specify the number of lines to be accumulated in the buffer
        u32 transferLine = nn::camera::GetMaxLines( IMG_WIDTH, IMG_HEIGHT );

        if ( IMG_WIDTH < 16 ) // Because the GetMaxLines function gets the maximum number of lines, the original number of lines may be less than that.
        {
            transferLine = IMG_HEIGHT;
        }

        //  Sharpness
        nn::camera::SetSharpness( nn::camera::SELECT_OUT1, 3 );

        //  Exposure
        nn::camera::SetExposure( nn::camera::SELECT_OUT1, 3 );

        // Change the frame rate
        nn::camera::SetFrameRate( nn::camera::SELECT_OUT1, nn::camera::FRAME_RATE_30 );

        // Change image size
        nn::camera::SetSize( nn::camera::SELECT_OUT1, CAMERA_SIZE, nn::camera::CONTEXT_A );

        // Put camera in startup state
        nn::camera::Activate( nn::camera::SELECT_OUT1 );

        //  Launch camera process
        m_Process.Initialize( nn::camera::PORT_CAM1, transferLine, &s_AppHeap );

        //  Finalization event
        m_EndEvent.Initialize( false );

        //  Start camera process
        m_Process.StartThread();
    }
    //=====================================================================
    bool CameraModule_cl::Wait()
    {
        nn::os::WaitObject *    pa_WaitObjects[ 2 ];
        pa_WaitObjects[ 0 ] = m_Process.GetpCapturedEvent();
        pa_WaitObjects[ 1 ] = &m_EndEvent;

        s32 id = nn::os::WaitObject::WaitAny( pa_WaitObjects, 2 );

        if ( id == 1 )
            return false;

        //  Lock newest capture results
        m_Process.LockImage();
        return true;
    }
    //=====================================================================
    void CameraModule_cl::Finalize()
    {
        m_Process.Finalize();
        m_EndEvent.Signal();

        nn::camera::Finalize();
    }

    CameraModule_cl s_CameraModule;

    //=====================================================================
    /* Please see man pages for details 
     
     */
    class ARProcess_cl
    {
    public:
        ARProcess_cl()
             : m_Displaying( 0 ), m_Processing( 0 ), m_Displayable( false )
        {}

        void    Initialize();
        void    Finalize();
        bool    IsDisplayable() { return m_Displayable; }
        void    DisplayDone() { m_Displayable = false; }

        Photo_cl &      GetrDisplayablePhoto()           { return ma_Photo[ m_Displaying ]; }
        Photo_cl &      GetrDisplayableInfo()            { return ma_Info[ m_Displaying ]; }
        mw::nar::MarkerList_ac * GetpDisplayableMarkerList()     { return mpa_MarkerLists[ m_Displaying ]; }
        nn::fnd::TimeSpan & GetrDisplayableTransferTime()    { return ma_TransferTime[ m_Displaying ]; }
        nn::fnd::TimeSpan & GetrDisplayableARTime()          { return ma_ARTime[ m_Displaying ]; }
        void            UpdateMarkerTemplate( int idx );

        /* Please see man pages for details 
         */
        void    DrawPattern();
    private:
        s8          m_Displaying;
        s8          m_Processing;
        bool        m_Displayable;
        u8          a_Pad[1];

        Photo_cl            ma_Photo[ 2 ];
        Photo_cl            ma_Info[ 2 ];
        mw::nar::MarkerList_ac *     mpa_MarkerLists[ 2 ];
        nn::os::Thread          m_Thread;
        u32                     pad;
        nn::fnd::TimeSpan       ma_TransferTime[ 2 ];
        nn::fnd::TimeSpan       ma_ARTime[ 2 ];
        //  Work to create marker
        mw::nar::MarkerMakerWork_tc< 5 >     ma_MakerWorks[ 2 ];

        static void hook( ARProcess_cl * p )
        {
            p->process();
        }
        void    process();
    };
    //=====================================================================
    void ARProcess_cl::Initialize()
    {
        for ( int idx = 0; idx < 2; idx++ )
        {
            ma_Photo[ idx ].Initialize( &s_RenderSystem, &s_AppHeap );
            ma_Info [ idx ].Initialize( &s_RenderSystem, &s_AppHeap );
        }

        //  Lower than the main thread
        m_Thread.StartUsingAutoStack( hook, this, 4096, nn::os::Thread::GetCurrentPriority() + 2 );
    }
    //=====================================================================
    void ARProcess_cl::process()
    {
        mw::nar::MarkerDatabase_cl   narDatabase;
        mw::nar::Image_cl            narImage( IMG_WIDTH, IMG_HEIGHT, sa_NarImageWork ) ;

        //  For white marker designs with black frames, uncomment the following
        //narImage.SetBorder( mw::nar::e_BorderBlackWhite );
        
        narDatabase.Register( s_MarkerTemplate ) ;

        while ( true )
        {
            //  Wait for images from camera
            if ( ! s_CameraModule.Wait() )
                break;

            //  Run Y2R conversion in parallel

            //  Register Y2R job
            Y2RJob_cl   job;

            CameraProcess_cl & r_Process = s_CameraModule.GetrCameraProcess();

            job.Set( r_Process.GetpLockedYuv(), ma_Photo[ m_Processing ].GetpImage() );
            Y2RProcess_cl::ss().Push( job );

            //  Start Y2R conversion
            Y2RProcess_cl::ss().Start();

            //  AR marker detection
            nn::fnd::TimeSpan   ARBegin = nn::os::Tick::GetSystemCurrent().ToTimeSpan();

            narImage.SetImage( r_Process.GetpLockedYuv() );
            narImage.SetDebugImage( ma_Info[ m_Processing ].GetpImage() );

            if ( s_MarkerMaker )
            {
                //  Marker pattern detection
                mw::nar::MarkerDetector_cl::Detect( narImage, narDatabase, s_MarkerData, ma_MakerWorks[ m_Processing ] );
            }
            else
            {
                //  Marker detection
                int markerNum = mw::nar::MarkerDetector_cl::Detect( narImage, narDatabase, s_MarkerData, s_DetectWork );

                //  Estimate AR marker coordinate
                for ( int idx = 0 ; idx < markerNum ; idx++ )
                {
                    mw::nar::Marker_cl * p_Marker = s_MarkerData.GetrDetectingMarkerList().GetpMarker( idx ) ;
                    //  Suppress shaking
                    p_Marker->SuppressShake( 4.f, 16.f );

                    if ( ! p_Marker->IsEstimated() )
                    {
                        // Estimate marker coordinate system
                        s_NarTrans.Estimate( *p_Marker, s_Projection );
                    }
                }
            }

            ma_ARTime[ m_Processing ]   = nn::os::Tick::GetSystemCurrent().ToTimeSpan() - ARBegin;

            //  Save transfer time
            ma_TransferTime[ m_Processing ] = r_Process.GetrTransferTimeOfLockedImage();
            //  Maintain pointer to detection results
            mpa_MarkerLists[ m_Processing ] = &( s_MarkerData.GetrDetectingMarkerList() );

            //  Wait for Y2R conversion
            Y2RProcess_cl::ss().WaitJobFinish();

            m_Displaying    = m_Processing;
            //  Next process index
            m_Processing ^= 0x1;

            m_Displayable   = true;
        }
    }

    //=====================================================================
    void ARProcess_cl::DrawPattern()
    {
        const f32   c_size = 48.0f;
        glDisable( GL_BLEND );
        for ( u32 idx = 0; idx < ma_MakerWorks[ m_Displaying ].GetPatternNum(); idx++ )
        {
            mw::nar::Vec2F_st    point( idx * ( c_size + 8.0f ) + 8.f, 50.0f );
            GetrDisplayablePhoto().DrawPattern( point, c_size, ma_MakerWorks[ m_Displaying ].GetpPoint( idx ) );
        }
        ma_MakerWorks[ m_Displaying ].Flush();
    }
    //=====================================================================
    void ARProcess_cl::UpdateMarkerTemplate( int idx )
    {
        s_MarkerTemplate.pattern    = ma_MakerWorks[ m_Displaying ].GetrPattern( idx );
        s_MarkerTemplate.pattern.Print();
    }
    //=====================================================================
    void ARProcess_cl::Finalize()
    {
        m_Thread.Join();
        m_Thread.Finalize();

        for ( int idx = 0; idx < 2; idx++ )
        {
            ma_Photo[ idx ].Finalize();
            ma_Info [ idx ].Finalize();
        }
    }

    
    ARProcess_cl    s_ARProcess;

    //=====================================================================
    /* Please see man pages for details 
     
     */
    void initialize_gfx()
    {
        s_HeapForGx = reinterpret_cast< uptr >( s_AppHeap.Allocate( c_gl_memory_size ) );

        // Allocates 8 MB memory in FCRAM for graphics
        s_RenderSystem.Initialize( s_HeapForGx, c_gl_memory_size );


        s_PgID = glCreateProgram();
        s_ShID = glCreateShader( GL_VERTEX_SHADER );

        size_t work_size    = nn::fs::GetRomRequiredMemorySize( 1, 1 );
        void * work         = s_AppHeap.Allocate( work_size );

        NN_UTIL_PANIC_IF_FAILED( nn::fs::MountRom( 1, 1, work, work_size ) );

        nn::fs::FileReader file( L"rom:/shader.shbin" );
        size_t fileSize = file.GetSize();
        void * p_Buf = s_AppHeap.Allocate( fileSize );

        s32 read = file.Read( p_Buf, fileSize );
        glShaderBinary( 1, &s_ShID, GL_PLATFORM_BINARY_DMP, p_Buf, read );
        file.Finalize();
        s_AppHeap.Free( p_Buf );

        s_AppHeap.Free( work );

        glAttachShader( s_PgID, s_ShID );
        glAttachShader( s_PgID, GL_DMP_FRAGMENT_SHADER_DMP );

        glBindAttribLocation( s_PgID, 0, "aPosition" );
        glBindAttribLocation( s_PgID, 1, "aColor" );

        glLinkProgram( s_PgID );
        glValidateProgram( s_PgID );
        glUseProgram( s_PgID );

        glClearColor( 0.36f, 0.42f, 0.5f, 1.0f );
        glClearDepthf( 1.f );

        glEnable( GL_DEPTH_TEST );
        glEnable( GL_SCISSOR_TEST );
        glEnable( GL_CULL_FACE );
        glDepthFunc( GL_LESS );

        s_RenderSystem.SetClearColor( NN_GX_DISPLAY0, 0.36f, 0.42f, 0.5f, 1.0f );
        s_RenderSystem.SetClearColor( NN_GX_DISPLAY1, 0.36f, 0.42f, 0.5f, 1.0f );

        glUniform3i( glGetUniformLocation( s_PgID, "dmp_TexEnv[0].srcRgb" ), GL_PRIMARY_COLOR, GL_PRIMARY_COLOR, GL_PRIMARY_COLOR );
        glUniform3i( glGetUniformLocation( s_PgID, "dmp_TexEnv[0].srcAlpha" ), GL_PRIMARY_COLOR, GL_PRIMARY_COLOR, GL_PRIMARY_COLOR );

    }

    //=====================================================================
    /* Please see man pages for details 
     
     */
    void finalize_gfx()
    {
        glDetachShader( s_PgID, GL_DMP_FRAGMENT_SHADER_DMP );
        glDetachShader( s_PgID, s_ShID );
        glDeleteShader( s_ShID );
        glDeleteProgram( s_PgID );

        s_RenderSystem.Finalize();
        s_AppHeap.Free( reinterpret_cast<void*>( s_HeapForGx ) );
    }

    //=====================================================================
    /* Please see man pages for details 
     
     */
    void initialize()
    {
        nn::Result result;

        nn::fs::Initialize();
        // Initialize hid
        result = nn::hid::Initialize();
        NN_UTIL_PANIC_IF_FAILED( result );

        // Create heap
        s_AppHeap.Initialize( nn::os::GetDeviceMemoryAddress(),
                              nn::os::GetDeviceMemorySize(),
                              nn::os::ALLOCATE_OPTION_LINEAR );

        // GX initialization
        initialize_gfx();

        //  Initialize camera
        s_CameraModule.Initialize();

        //  Configure projection
        nn::camera::StereoCameraCalibrationData carib;
        nn::camera::GetStereoCameraCalibrationData( &carib );
        s_Projection.Set( IMG_WIDTH, IMG_HEIGHT, NEAR, FAR, carib.angleOfViewRight );

        //  Initialize Y2R
        Y2RProcess_cl::ss().Initialize();

        //  AR processing
        s_ARProcess.Initialize();
    }

    //=====================================================================
    /* Please see man pages for details 
     
     */
    void finalize()
    {
        nn::hid::Finalize();
        s_CameraModule.Finalize();
        s_ARProcess.Finalize();
        Y2RProcess_cl::ss().Finalize();
        
        finalize_gfx();
        s_AppHeap.Finalize();       
    }

    //=====================================================================
    /* Please see man pages for details 
     
     */
    void processPad( nn::hid::PadReader &r_Reader, nn::hid::PadStatus & r_Status )
    {
        //  Load pad
        r_Reader.ReadLatest( &r_Status );

        if ( r_Status.trigger & nn::hid::BUTTON_X )
        {
            s_ViewInfo = ! s_ViewInfo;

            mw::nar::Debug_cl::Switch_e  viewInfo = static_cast< mw::nar::Debug_cl::Switch_e >( mw::nar::Debug_cl::me_ViewClear
                                                                              | mw::nar::Debug_cl::me_ViewLineCandidates
                                                                              | mw::nar::Debug_cl::me_ViewCheckMesh
                                                                              | mw::nar::Debug_cl::me_ViewEdgeSlope ) ;
            if ( s_ViewInfo )
            {
                mw::nar::Debug_cl::GetrSwitch().TurnOn( viewInfo ) ;
            }
            else
            {
                mw::nar::Debug_cl::GetrSwitch().TurnOff( viewInfo ) ;
            }
        }

        if ( r_Status.trigger & nn::hid::BUTTON_Y )
        {
            s_MarkerMaker = ! s_MarkerMaker;
        }

        if ( s_MarkerMaker )
        {
            if      ( r_Status.trigger & nn::hid::BUTTON_A )
            {
                s_ARProcess.UpdateMarkerTemplate( s_MarkerIdx );
            }
            else
            {
                if      ( r_Status.trigger & nn::hid::BUTTON_LEFT )
                {
                    if ( --s_MarkerIdx >= c_MakerWorkSize ) s_MarkerIdx = c_MakerWorkSize-1;
                }
                else if ( r_Status.trigger & nn::hid::BUTTON_RIGHT )
                {
                    if ( ++s_MarkerIdx >= c_MakerWorkSize ) s_MarkerIdx = 0;
                }
            }

        }
        else
        {
            if ( s_ViewInfo )
            {
                if      ( r_Status.trigger & nn::hid::BUTTON_UP )
                {
                    mw::nar::Debug_cl::GetrSwitch().Toggle( mw::nar::Debug_cl::me_ViewLineCandidates ) ;
                }
                else if ( r_Status.trigger & nn::hid::BUTTON_LEFT )
                {
                    mw::nar::Debug_cl::GetrSwitch().Toggle( mw::nar::Debug_cl::me_ViewCheckMesh ) ;
                }
                else if ( r_Status.trigger & nn::hid::BUTTON_RIGHT )
                {
                    mw::nar::Debug_cl::GetrSwitch().Toggle( mw::nar::Debug_cl::me_ViewEdgeSlope ) ;
                }
            }
        }
    }
}

//=====================================================================
/* Please see man pages for details 
 
 */
extern "C" void
nnMain()
{
    // Call only the nn::applet::Enable function to also allow execution from the HOME Menu
    // HOME Menu transitions, POWER Button presses, and sleep are all unsupported
    nn::applet::Enable(false);
    
    // Initialization
    initialize();

    //  Interpolate with previous matrix
    s_NarTrans.SetMildRotation( true, 0.9f );
    //  When not shaking, follow quickly
    s_NarTrans.SetAccelerationOnMildRotation( true, 0.25f );

    nn::math::MTX44   projMtx;
    s_Projection.GetMTX44( projMtx );

    nn::math::MTX34   transMtx;
    nn::math::VEC3    transVec( 0.f, 2.f, 0.f );
    nn::math::MTX34Translate( &transMtx, &transVec );

    nn::math::MTX34   scaleMtx;
    nn::math::VEC3    scaleVec( 2.f, 2.f, 2.f );
    nn::math::MTX34Scale( &scaleMtx, &scaleVec );

    nn::fnd::TimeSpan frameTS;
    nn::fnd::TimeSpan AREndTS,ARRateTS,ARLastTS;

    // Prepare pad
    nn::hid::PadReader   padReader;
    nn::hid::PadStatus   padStatus;

    int markerNum = 0;

    bool loop = true;
    while ( loop )
    {
        nn::fnd::TimeSpan   frameBegin  = nn::os::Tick::GetSystemCurrent().ToTimeSpan();

        //  Pad processing
        processPad( padReader, padStatus );

        if ( padStatus.trigger & nn::hid::BUTTON_START )
        {
            break;
        }

        //  AR render data ready
        if ( s_ARProcess.IsDisplayable() )
        {
            //  NOTE: SetRenderTarget calls SetViewport internally.
            s_RenderSystem.SetRenderTarget( NN_GX_DISPLAY0 );
            s_RenderSystem.Clear();
            s_RenderSystem.SetViewport( 0, 40, 240, 320 );
            glScissor( 0, 40, 240, 320 );

            glEnable( GL_BLEND );
            glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );

            //  Camera image
            s_ARProcess.GetrDisplayablePhoto().Draw();

            if ( s_ViewInfo )
            {
                //  Debugging information
                s_ARProcess.GetrDisplayableInfo().Draw();
            }

            mw::nar::MarkerList_ac * p_List = s_ARProcess.GetpDisplayableMarkerList();

            glUseProgram( s_PgID );

            //  Projection matrix
            glUniformMatrix4fv( glGetUniformLocation( s_PgID, "uProjection" ), 1, GL_TRUE, static_cast< f32 * >( projMtx ) );

                //  Marker render
            markerNum = p_List->GetMarkerNum();
            if ( markerNum > 0 )
            {
                for ( int idx = 0 ; idx < markerNum ; idx++ )
                {
                    mw::nar::Marker_cl * p_Marker = p_List->GetpMarker( idx ) ;

                    if ( p_Marker && p_Marker->IsEstimated() )
                    {
                        //  Model view matrix
                        nn::math::MTX34   mtx;
                        p_Marker->GetrTransformation().GetMTX34( mtx );

                        nn::math::MTX34Mult( &mtx, &mtx, &transMtx );
                        nn::math::MTX34Mult( &mtx, &mtx, &scaleMtx );
                        nn::math::MTX44   mv( mtx );
                        glUniformMatrix4fv( glGetUniformLocation( s_PgID, "uModelView" ), 1, GL_TRUE, static_cast< f32 * >( mv ) );

                        Cube_cl::draw();
                    }
                }
            }

            s_RenderSystem.SwapBuffers();

            //  Rendering ended
            s_ARProcess.DisplayDone();

            AREndTS     = nn::os::Tick::GetSystemCurrent().ToTimeSpan();
            ARRateTS    = AREndTS - ARLastTS;
            ARLastTS    = AREndTS;

            s_RenderSystem.SetRenderTarget( NN_GX_DISPLAY1 );
            s_RenderSystem.Clear();

            s_RenderSystem.SetColor( 1.0f, 1.0f, 1.0f );
            s_RenderSystem.SetFontSize( 8.0f );
            s_RenderSystem.DrawText( 10.0f, 10.0f, "= Marker Maker =" );
            s_RenderSystem.DrawText( 10.0f, 20.0f, "Marker Patten %d", s_MarkerIdx );
            s_RenderSystem.DrawText( 10.0f, 30.0f, "Select:<+> Update:A" );

            s_RenderSystem.DrawText( 10.0f, 190.0f, "    cam: %6lldus", s_ARProcess.GetrDisplayableTransferTime().GetMicroSeconds() );
            s_RenderSystem.DrawText( 10.0f, 200.0f, "     AR: %6lldus", s_ARProcess.GetrDisplayableARTime().GetMicroSeconds() );
            s_RenderSystem.DrawText( 10.0f, 210.0f, "  frame: %6lldus", frameTS.GetMicroSeconds() );
            s_RenderSystem.DrawText( 10.0f, 220.0f, "AR rate: %6lldus", ARRateTS.GetMicroSeconds() );

            for ( int idx = 0 ; idx < markerNum ; idx++ )
            {
                mw::nar::Marker_cl * p_Marker = p_List->GetpMarker( idx ) ;
                s_RenderSystem.DrawText( 10.0f, 90.0f + 10.0f * idx, "marker id:%d: score:%f", p_Marker->GetID(), p_Marker->GetScore() );
            }

            s_ARProcess.DrawPattern();

            s_RenderSystem.SwapBuffers();
        }
        frameTS = nn::os::Tick::GetSystemCurrent().ToTimeSpan() - frameBegin;

        s_RenderSystem.WaitVsync( NN_GX_DISPLAY_BOTH );
    }

    //  End processing
    finalize();

}

