/*-----------------------------------------------------------------------*
  Project:  Nintendo Augmented Reality Library.
  File:     sampledemos\common\cube.cpp

  Copyright (C)2011 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensed developers
  and are protected by national and international copyright laws. They
  may not be disclosed to third parties or copied or duplicated in any
  form, in whole or in part, without the prior written consent of
  Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
*-----------------------------------------------------------------------*/

/* Please see man pages for details 
 
 
 */

#include <nn.h>
#include <nn/gx.h>
#include "cube.h"

//=====================================================================
namespace
{
    const GLfloat sca_coords[] =
    {
        -1.f, -1.f, -1.f, 1.f,
        -1.f, -1.f,  1.f, 1.f,
        -1.f,  1.f, -1.f, 1.f,
        -1.f,  1.f,  1.f, 1.f,
         1.f, -1.f, -1.f, 1.f,
         1.f, -1.f,  1.f, 1.f,
         1.f,  1.f, -1.f, 1.f,
         1.f,  1.f,  1.f, 1.f,
    };

    const GLfloat sca_colors[] =
    {
        0.f, 0.f, 0.f,
        1.f, 0.f, 0.f,
        0.f, 0.f, 1.f,
        1.f, 0.f, 1.f,
        0.f, 1.f, 0.f,
        1.f, 1.f, 0.f,
        0.f, 1.f, 1.f,
        1.f, 1.f, 1.f,
    };

    /**
     *      2 --- 6
     *    / |   / |
     *  3 --- 7   |
     *  |   0 |-- 4
     *  | /   | /
     *  1 --- 5
     *
     *  0 - 2
     *  | / |   left
     *  1 - 3
     *  | / |   front
     *  5 - 7
     *  | / |   right
     *  4 - 6
     *  | / |   back
     *  0 - 2
     *  | / |   warp
     *  2 - 6
     *  | / |   top
     *  3 - 7
     *  | / |   warp
     *  7 - 1
     *  | / |   warp
     *  1 - 5
     *  | / |   bottom
     *  0 - 4
     */
    const GLushort sca_indices[] =
    {
        0, 2,
        1, 3,
        5, 7,
        4, 6,
        0, 2,
        2, 6,
        3, 7,
        7, 1,
        1, 5,
        0, 4
    };

    GLuint s_ArrayBufferID          = 0;
    GLuint s_ElementArrayBufferID   = 0;
}

//=====================================================================
void Cube_cl::draw()
{
    if ( s_ArrayBufferID == 0 )
    {
        glGenBuffers( 1, &s_ArrayBufferID );
        glBindBuffer( GL_ARRAY_BUFFER, s_ArrayBufferID );
        glBufferData( GL_ARRAY_BUFFER, sizeof( sca_coords ) + sizeof( sca_colors ), 0, GL_STATIC_DRAW );    //  Only allocate region
        glBufferSubData( GL_ARRAY_BUFFER, 0, sizeof( sca_coords ), sca_coords );
        glBufferSubData( GL_ARRAY_BUFFER, sizeof( sca_coords ), sizeof( sca_colors ), sca_colors );
        glGenBuffers( 1, &s_ElementArrayBufferID );
        glBindBuffer( GL_ELEMENT_ARRAY_BUFFER, s_ElementArrayBufferID );
        glBufferData( GL_ELEMENT_ARRAY_BUFFER, sizeof( sca_indices ), sca_indices, GL_STATIC_DRAW );

    }
    else
    {
        glBindBuffer( GL_ARRAY_BUFFER, s_ArrayBufferID );
        glBufferData( GL_ARRAY_BUFFER, sizeof( sca_coords ) + sizeof( sca_colors ), 0, GL_STATIC_DRAW );    //  Only allocate region
        glBufferSubData( GL_ARRAY_BUFFER, 0, sizeof( sca_coords ), sca_coords );
        glBufferSubData( GL_ARRAY_BUFFER, sizeof( sca_coords ), sizeof( sca_colors ), sca_colors );
        glBindBuffer( GL_ELEMENT_ARRAY_BUFFER, s_ElementArrayBufferID );
        glBufferData( GL_ELEMENT_ARRAY_BUFFER, sizeof( sca_indices ), sca_indices, GL_STATIC_DRAW );
    }

    glEnableVertexAttribArray( 0 );
    glEnableVertexAttribArray( 1 );

    glVertexAttribPointer( 0, 4, GL_FLOAT, GL_FALSE, 0, 0 );
    glVertexAttribPointer( 1, 3, GL_FLOAT, GL_FALSE, 0, ( GLvoid * )sizeof( sca_coords ) );

    glFrontFace( GL_CW );
    glEnable( GL_DEPTH_TEST );
    glDepthFunc( GL_LESS );
    glDrawElements( GL_TRIANGLE_STRIP, sizeof( sca_indices )/sizeof( sca_indices[ 0 ] ), GL_UNSIGNED_SHORT, 0 );
    glFrontFace( GL_CCW );
}
