/*-----------------------------------------------------------------------*
  Project:  Nintendo Augmented Reality Library.
  File:     sampledemos\common\narDemoPhoto.cpp

  Copyright (C)2011 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensed developers
  and are protected by national and international copyright laws. They
  may not be disclosed to third parties or copied or duplicated in any
  form, in whole or in part, without the prior written consent of
  Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
*-----------------------------------------------------------------------*/

/* Please see man pages for details 
 
 
 */
#include <nn.h>
#include <nn/fnd.h>

#include "narVec2.h"

#include "narDemoConfig.h"
#include "narDemoPhoto.h"

#include "demo.h"

using namespace nn;

//=====================================================================
void Photo_cl::Initialize( ::demo::RenderSystemDrawing * p_RenderSystem, fnd::ExpHeap * p_Heap )
{
    mp_RenderSsytem = p_RenderSystem;
    mp_Heap         = p_Heap;
    map_Image       = reinterpret_cast< u16 * >( mp_Heap->Allocate( TEXTURE_BYTES, 64 ) );
}
//=====================================================================
void Photo_cl::Draw()
{
    Draw( 0.f, 0.f, 1.f );
}

//=====================================================================
void Photo_cl::Draw( const f32 windowCoordinateX, const f32 windowCoordinateY, const f32 scale )
{
    if ( m_TextureId == 0 )
    {
        mp_RenderSsytem->GenerateTexture( GL_TEXTURE_2D | NN_GX_MEM_FCRAM | GL_NO_COPY_FCRAM_DMP,
                              GL_RGBA_NATIVE_DMP,
                              TEXTURE_WIDTH, TEXTURE_HEIGHT,
                              GL_RGBA_NATIVE_DMP,
                              GL_UNSIGNED_SHORT_5_5_5_1,
                              map_Image,
                              m_TextureId );
    }

    //  Create texture using squares
    mp_RenderSsytem->FillTexturedRectangle( m_TextureId,
                                windowCoordinateX, windowCoordinateY,
                                320.f * scale, 320.f * scale,
                                TEXTURE_WIDTH, TEXTURE_HEIGHT,
                                TEXTURE_WIDTH, TEXTURE_HEIGHT );
}


//=====================================================================
void Photo_cl::DrawPattern( const mw::nar::Vec2F_st & r_p0, f32 size, const mw::nar::Vec2F_st * cap_Points )
{
    f32 p1x = r_p0.X() + size;
    f32 p1y = r_p0.Y() + size;
    mp_RenderSsytem->FillTexturedTriangle( m_TextureId,
                               r_p0.X(), r_p0.Y(),
                               cap_Points[ 0 ].X() / TEXTURE_WIDTH, 1.f - cap_Points[ 0 ].Y() / TEXTURE_HEIGHT,
                               r_p0.X(), p1y,
                               cap_Points[ 1 ].X() / TEXTURE_WIDTH, 1.f - cap_Points[ 1 ].Y() / TEXTURE_HEIGHT,
                               p1x, p1y,
                               cap_Points[ 2 ].X() / TEXTURE_WIDTH, 1.f - cap_Points[ 2 ].Y() / TEXTURE_HEIGHT );

    mp_RenderSsytem->FillTexturedTriangle( m_TextureId,
                               r_p0.X(), r_p0.Y(),
                               cap_Points[ 0 ].X() / TEXTURE_WIDTH, 1.f - cap_Points[ 0 ].Y() / TEXTURE_HEIGHT,
                               p1x, p1y,
                               cap_Points[ 2 ].X() / TEXTURE_WIDTH, 1.f - cap_Points[ 2 ].Y() / TEXTURE_HEIGHT,
                               p1x, r_p0.Y(),
                               cap_Points[ 3 ].X() / TEXTURE_WIDTH, 1.f - cap_Points[ 3 ].Y() / TEXTURE_HEIGHT );
}

//=====================================================================
void Photo_cl::Finalize()
{
    if ( m_TextureId != 0 )
    {
        mp_RenderSsytem->DeleteTexture( m_TextureId );
    }
    mp_Heap->Free( reinterpret_cast< void * >( map_Image ) );
}
