/*-----------------------------------------------------------------------*
  Project:  Nintendo Augmented Reality Library.
  File:     sampledemos\stereo\texturedRectangle.h

  Copyright (C)2011 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensed developers
  and are protected by national and international copyright laws. They
  may not be disclosed to third parties or copied or duplicated in any
  form, in whole or in part, without the prior written consent of
  Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
*-----------------------------------------------------------------------*/

/* Please see man pages for details 
 
 
 */

#include "texturedRectangle.h"

//=====================================================================
namespace
{
    // Texture object (2 for left and right)
    GLuint  sa_Texture[ 2 ] = { 0, 0 };

    // VBO
    GLuint  sa_BufferID[ 2 ] = { 0, 0 };

    // Vertex attributes (position)
    const GLfloat sa_Position[ 4 * 4 ] =
    {
         1.0f,  1.0f,  0.0f,  1.0f, // upper right
        -1.0f,  1.0f,  0.0f,  1.0f, // upper left
        -1.0f, -1.0f,  0.0f,  1.0f, // lower left
         1.0f, -1.0f,  0.0f,  1.0f, // lower right
    };

    // Vertex attributes (texture coordinates)
    const GLfloat sa_TexCoord[ 2 * 4 ] =
    {
        1.0f, 1.0f,  // upper right
        0.0f, 1.0f,  // upper left
        0.0f, 0.0f,  // lower left
        1.0f, 0.0f,  // lower right
    };

    // Vertex index
    const GLushort sa_Index[ 4 ] = { 0, 1, 3, 2 };

} //Namespace


//=====================================================================
void TexturedRectangle_cl::draw(
    const int eye,
    const void * p_Image,
    const int width,
    const int height )
{
    // Texture settings

    if( sa_Texture[ 0 ] == 0 )
    {
        glGenTextures( 2, sa_Texture );
    }

    glBindTexture( GL_TEXTURE_2D, sa_Texture[ eye ] );

    glTexImage2D( GL_TEXTURE_2D | NN_GX_MEM_FCRAM | GL_NO_COPY_FCRAM_DMP,
                  0,  // MipLevel
                  GL_RGBA_NATIVE_DMP,
                  width,
                  height,
                  0,  // always 0
                  GL_RGBA_NATIVE_DMP,
                  GL_UNSIGNED_SHORT_5_5_5_1,
                  p_Image );

    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );

    // VBO settings

    if( sa_BufferID[ 0 ] == 0 )
    {
        glGenBuffers( 2, sa_BufferID );
        glBindBuffer( GL_ARRAY_BUFFER, sa_BufferID[ 0 ] );
        glBufferData( GL_ARRAY_BUFFER, sizeof( sa_Position ) + sizeof( sa_TexCoord ), 0, GL_STATIC_DRAW );
        glBufferSubData( GL_ARRAY_BUFFER, 0, sizeof( sa_Position ), sa_Position );
        glBufferSubData( GL_ARRAY_BUFFER, sizeof( sa_Position ), sizeof( sa_TexCoord ), sa_TexCoord );

        glBindBuffer( GL_ELEMENT_ARRAY_BUFFER, sa_BufferID[ 1 ] );
        glBufferData( GL_ELEMENT_ARRAY_BUFFER, sizeof( sa_Index ), sa_Index, GL_STATIC_DRAW );
    }
    else
    {
        glBindBuffer( GL_ARRAY_BUFFER, sa_BufferID[ 0 ] );
        glBindBuffer( GL_ELEMENT_ARRAY_BUFFER, sa_BufferID[ 1 ] );
    }

    glEnableVertexAttribArray( 0 );
    glEnableVertexAttribArray( 1 );

    // Make it so the vertex attribute location is decided by the shader
    glVertexAttribPointer( 0, 4, GL_FLOAT, GL_FALSE, 0, 0 );
    glVertexAttribPointer( 1, 2, GL_FLOAT, GL_FALSE, 0, ( void * )( sizeof( sa_Position ) ) );

    // Render kick

    glDrawElements( GL_TRIANGLE_STRIP, 4, GL_UNSIGNED_SHORT, 0 );

} //TexturedRectangle_cl::Draw()
